@@ -4,7 +4,7 @@ keys = gl.keys.list()
 
 # global get
 key = gl.keys.get(key_id)
-# end global key
+# end global get
 
 # list
 keys = gl.project_keys.list(project_id=1)
@@ -36,9 +36,9 @@ key.delete()
 # end delete
 
 # enable
-deploy_key.enable()
+project.keys.enable(key_id)
 # end enable
 
 # disable
-deploy_key.disable()
+project.keys.disable(key_id)
 # end disable
