@@ -99,6 +99,8 @@ class BaseManager(object):
         args = self._set_parent_args(**kwargs)
         if not self.obj_cls.canGet:
             raise NotImplementedError
+        if id is None and self.obj_cls.getRequiresId is True:
+            raise ValueError('The id argument must be defined.')
         return self.obj_cls.get(self.gitlab, id, **args)
 
     def list(self, **kwargs):
@@ -220,9 +222,10 @@ class GitlabObject(object):
         for attribute in attributes:
             if hasattr(self, attribute):
                 value = getattr(self, attribute)
-                if isinstance(value, list):
-                    value = ",".join(value)
-                if attribute == 'sudo':
+                # labels need to be sent as a comma-separated list
+                if attribute == 'labels' and isinstance(value, list):
+                    value = ", ".join(value)
+                elif attribute == 'sudo':
                     value = str(value)
                 data[attribute] = value
 
@@ -734,6 +737,7 @@ class CurrentUser(GitlabObject):
 class ApplicationSettings(GitlabObject):
     _url = '/application/settings'
     _id_in_update_url = False
+    getRequiresId = False
     optionalUpdateAttrs = ['after_sign_out_path',
                            'container_registry_token_expire_delay',
                            'default_branch_protection',
@@ -760,6 +764,15 @@ class ApplicationSettings(GitlabObject):
     canCreate = False
     canDelete = False
 
+    def _data_for_gitlab(self, extra_parameters={}, update=False,
+                         as_json=True):
+        data = (super(ApplicationSettings, self)
+                ._data_for_gitlab(extra_parameters, update=update,
+                                  as_json=False))
+        if not self.domain_whitelist:
+            data.pop('domain_whitelist', None)
+        return json.dumps(data)
+
 
 class ApplicationSettingsManager(BaseManager):
     obj_cls = ApplicationSettings
@@ -792,6 +805,7 @@ class KeyManager(BaseManager):
 class NotificationSettings(GitlabObject):
     _url = '/notification_settings'
     _id_in_update_url = False
+    getRequiresId = False
     optionalUpdateAttrs = ['level',
                            'notification_email',
                            'new_note',
@@ -856,7 +870,9 @@ class GroupMember(GitlabObject):
     canGet = 'from_list'
     requiredUrlAttrs = ['group_id']
     requiredCreateAttrs = ['access_level', 'user_id']
+    optionalCreateAttrs = ['expires_at']
     requiredUpdateAttrs = ['access_level']
+    optionalCreateAttrs = ['expires_at']
     shortPrintAttr = 'username'
 
     def _update(self, **kwargs):
@@ -1274,8 +1290,9 @@ class ProjectCommit(GitlabObject):
     _url = '/projects/%(project_id)s/repository/commits'
     canDelete = False
     canUpdate = False
-    canCreate = False
     requiredUrlAttrs = ['project_id']
+    requiredCreateAttrs = ['branch_name', 'commit_message', 'actions']
+    optionalCreateAttrs = ['author_email', 'author_name']
     shortPrintAttr = 'title'
     managers = (
         ('comments', ProjectCommitCommentManager,
@@ -1360,23 +1377,22 @@ class ProjectKey(GitlabObject):
     requiredUrlAttrs = ['project_id']
     requiredCreateAttrs = ['title', 'key']
 
-    def enable(self):
-        """Enable a deploy key for a project."""
-        url = '/projects/%s/deploy_keys/%s/enable' % (self.project_id, self.id)
-        r = self.gitlab._raw_post(url)
-        raise_error_from_response(r, GitlabProjectDeployKeyError, 201)
-
-    def disable(self):
-        """Disable a deploy key for a project."""
-        url = '/projects/%s/deploy_keys/%s/disable' % (self.project_id,
-                                                       self.id)
-        r = self.gitlab._raw_delete(url)
-        raise_error_from_response(r, GitlabProjectDeployKeyError, 200)
-
 
 class ProjectKeyManager(BaseManager):
     obj_cls = ProjectKey
 
+    def enable(self, key_id):
+        """Enable a deploy key for a project."""
+        url = '/projects/%s/deploy_keys/%s/enable' % (self.parent.id, key_id)
+        r = self.gitlab._raw_post(url)
+        raise_error_from_response(r, GitlabProjectDeployKeyError, 201)
+
+    def disable(self, key_id):
+        """Disable a deploy key for a project."""
+        url = '/projects/%s/deploy_keys/%s/disable' % (self.parent.id, key_id)
+        r = self.gitlab._raw_delete(url)
+        raise_error_from_response(r, GitlabProjectDeployKeyError, 200)
+
 
 class ProjectEvent(GitlabObject):
     _url = '/projects/%(project_id)s/events'
@@ -1412,7 +1428,8 @@ class ProjectHook(GitlabObject):
     requiredCreateAttrs = ['url']
     optionalCreateAttrs = ['push_events', 'issues_events', 'note_events',
                            'merge_requests_events', 'tag_push_events',
-                           'build_events', 'enable_ssl_verification', 'token']
+                           'build_events', 'enable_ssl_verification', 'token',
+                           'pipeline_events']
     shortPrintAttr = 'url'
 
 
@@ -1442,29 +1459,16 @@ class ProjectIssue(GitlabObject):
     requiredUrlAttrs = ['project_id']
     requiredCreateAttrs = ['title']
     optionalCreateAttrs = ['description', 'assignee_id', 'milestone_id',
-                           'labels', 'created_at']
+                           'labels', 'created_at', 'due_date']
     optionalUpdateAttrs = ['title', 'description', 'assignee_id',
                            'milestone_id', 'labels', 'created_at',
-                           'updated_at', 'state_event']
+                           'updated_at', 'state_event', 'due_date']
     shortPrintAttr = 'title'
     managers = (
         ('notes', ProjectIssueNoteManager,
             [('project_id', 'project_id'), ('issue_id', 'id')]),
     )
 
-    def _data_for_gitlab(self, extra_parameters={}, update=False,
-                         as_json=True):
-        # Gitlab-api returns labels in a json list and takes them in a
-        # comma separated list.
-        if hasattr(self, "labels"):
-            if (self.labels is not None and
-               not isinstance(self.labels, six.string_types)):
-                labels = ", ".join(self.labels)
-                extra_parameters['labels'] = labels
-
-        return super(ProjectIssue, self)._data_for_gitlab(extra_parameters,
-                                                          update)
-
     def subscribe(self, **kwargs):
         """Subscribe to an issue.
 
@@ -1528,7 +1532,9 @@ class ProjectMember(GitlabObject):
     _url = '/projects/%(project_id)s/members'
     requiredUrlAttrs = ['project_id']
     requiredCreateAttrs = ['access_level', 'user_id']
+    optionalCreateAttrs = ['expires_at']
     requiredUpdateAttrs = ['access_level']
+    optionalCreateAttrs = ['expires_at']
     shortPrintAttr = 'username'
 
 
@@ -2020,6 +2026,7 @@ class ProjectService(GitlabObject):
     canCreate = False
     _id_in_update_url = False
     _id_in_delete_url = False
+    getRequiresId = False
     requiredUrlAttrs = ['project_id', 'service_name']
 
     _service_attrs = {
@@ -2035,6 +2042,10 @@ class ProjectService(GitlabObject):
         'drone-ci': (('token', 'drone_url'), ('enable_ssl_verification', )),
         'emails-on-push': (('recipients', ), ('disable_diffs',
                                               'send_from_committer_email')),
+        'builds-email': (('recipients', ), ('add_pusher',
+                                            'notify_only_broken_builds')),
+        'pipelines-email': (('recipients', ), ('add_pusher',
+                                               'notify_only_broken_builds')),
         'external-wiki': (('external_wiki_url', ), tuple()),
         'flowdock': (('token', ), tuple()),
         'gemnasium': (('api_key', 'token', ), tuple()),
@@ -2137,6 +2148,16 @@ class ProjectDeploymentManager(BaseManager):
     obj_cls = ProjectDeployment
 
 
+class ProjectRunner(GitlabObject):
+    _url = '/projects/%(project_id)s/runners'
+    canUpdate = False
+    requiredCreateAttrs = ['runner_id']
+
+
+class ProjectRunnerManager(BaseManager):
+    obj_cls = ProjectRunner
+
+
 class Project(GitlabObject):
     _url = '/projects'
     _constructorTypes = {'owner': 'User', 'namespace': 'Group'}
@@ -2184,6 +2205,7 @@ class Project(GitlabObject):
         ('notificationsettings', ProjectNotificationSettingsManager,
          [('project_id', 'id')]),
         ('pipelines', ProjectPipelineManager, [('project_id', 'id')]),
+        ('runners', ProjectRunnerManager, [('project_id', 'id')]),
         ('services', ProjectServiceManager, [('project_id', 'id')]),
         ('snippets', ProjectSnippetManager, [('project_id', 'id')]),
         ('tags', ProjectTagManager, [('project_id', 'id')]),
@@ -2473,6 +2495,7 @@ class Runner(GitlabObject):
     _url = '/runners'
     canCreate = False
     optionalUpdateAttrs = ['description', 'active', 'tag_list']
+    optionalListAttrs = ['scope']
 
 
 class RunnerManager(BaseManager):
